/* File: dist_records.c
 * 
 * Hatem Nassrat
 * B00393388 - Dalhousie University
 *
 * Part of A Fault tolerance Implementation for an OLAP system
 *
 */

#include <mpi.h>
#include <stdio.h>
#include <strings.h>
#include <sys/time.h>
#include "defines.h"

#define IC_CREATE_TAG 5
#define hostfile "/tmp/pcc05/hostfile"

int main(int argc, char * argv[])
{
    int i,j,k, my_rank, p, f;
    int flag;
    char buf[BUFSIZ];
    MPI_Comm *my_comm;
    FILE *file;
    FILE **out;
    char **nodes,*temp;
    Element arecord;
    MPI_Status status;
    struct timeval start,stop;

    /*MPI INIT*/
    
    MPI_Init(&argc,&argv);
    MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &p);
    f=p;
    
    gettimeofday(&start,NULL);
    
       /* create intercommunicators and set error handlers */
    if(my_rank==0) //manager process
    {
        my_comm = (MPI_Comm*) malloc ((p-1)*sizeof(MPI_Comm));
        for(i=1;i<p;i++)
        {
            MPI_Intercomm_create(MPI_COMM_SELF, 0, MPI_COMM_WORLD, i, IC_CREATE_TAG, &my_comm[i-1]);
            MPI_Comm_set_errhandler(my_comm[i-1], MPI_ERRORS_RETURN);
        }
    }
    else
    {
        my_comm = (MPI_Comm*)malloc(sizeof(MPI_Comm));
        MPI_Intercomm_create(MPI_COMM_SELF, 0, MPI_COMM_WORLD, 0, IC_CREATE_TAG, my_comm);
        MPI_Comm_set_errhandler(*my_comm, MPI_ERRORS_RETURN);
    }
        /* Each p has a link to manager and vice versa */
 
    /*END OF: MPI INIT*/

    for(i=1;i<argc;i++)
        if(my_rank == atoi(argv[i])) exit(0);
        else 
        {
            f--;
            //closing communicator
            if(my_rank==0)
            {
                //MPI_Comm_free(&my_comm[atoi(argv[i])-1]);
                my_comm[atoi(argv[i])-1]=NULL;
            }
        }
        
    file = fopen(hostfile,"r");
    nodes = malloc(p*sizeof(char*));
    for(i=0;i<p;i++)
    {
        fgets(buf,BUFSIZ,file);
        buf[strlen(buf)-1]=0;
        nodes[i]=strdup(buf);
    }
    fclose(file);
    
    if(my_rank == 0)
    {
        out = (FILE**)malloc((f-1)*sizeof(FILE*));
        k=0;
        for(i=1;i<p;i++)
        {
            flag=0;
            for(j=1;j<argc;j++)
               if(atoi(argv[j])==i){flag=1;break;}
            if(!flag)  
            {
                sprintf(buf,"/tmp/pcc05/P%d-r",i);
                out[k] = fopen(buf,"w");
                k++;
            }
        }

        k=0;
        for(i=1;i<argc;i++)
        {
            sprintf(buf,"/tmp/pcc05/P%d",atoi(argv[i]));
            file = fopen(buf,"r");
            if(!file)
                perror(buf);
            else
            {
                //i=0;
                while(fscanf(file,"%d,%f\n",&(arecord.key),&(arecord.data))!=EOF)
                {//i++;
                
                    fprintf(out[k],"%d,%f\n",arecord.key,arecord.data);
                    k=(k+1)%(f-1);
                    //printf("%d,%f\n",arecord.key,arecord.data);
                    //if(i==51)exit(0);


                }
                fclose(file);
            }
        }

        
        for(k=0;k<f-1;k++)fclose(out[k]);
        for(i=1;i<p;i++)
            if(my_comm[i-1]) MPI_Send(&i,1,MPI_INT,0,0,my_comm[i-1]);
    }
    else
    {
        j=-1;
        MPI_Recv(&j,1,MPI_INT,0,0,*my_comm,&status);
        //printf("status %d",status.MPI_ERROR);
        if(j==-1) exit(0);
        sprintf(buf, "scp %s:/tmp/pcc05/P%d-r /tmp/pcc05/recovered_data",nodes[0],my_rank);
        //printf("%s\n",buf);
        system(buf);
    }

    
    gettimeofday(&stop,NULL);
    if(my_rank==1)
        printf("Dist records: time= %.4f\n",
                (stop.tv_sec-start.tv_sec)+
                ((float)(stop.tv_usec-start.tv_usec))/(1000000));
    
   /* Finalize */
    //if(my_rank==0) 
    //    for (i=1; i<p; i++) 
    //        if(my_comm[i-1]) MPI_Comm_free(&my_comm[i-1]);
    //else MPI_Comm_free(my_comm);
    MPI_Finalize();
    exit(0);
}
