/* File: MPI_Fault.c
 * 
 * Hatem Nassrat
 * B00393388 - Dalhousie University
 *
 * A Fault tolerance Implementation for an OLAP system
 *
 */

#include <mpi.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defines.h"

#define IC_CREATE_TAG 5


Element* local_load(char *filename, int n, char *rfile, int *size);
Element* get_by_key(Element *data, int size, int key, int* rsize);
Element* get_by_data(Element *data, int size, int key, int* rsize);

int main(int argc, char * argv[])
{
    int i, my_rank, p, f;
    int size,qcount;
    char buf_file[BUFSIZ];
    MPI_Comm *my_comm;
    Element* my_records;
    Element query, *results;
    MPI_Status status;
    int *qsize;
    FILE *qfile;
    struct timeval start,stop;


    /*MPI INIT*/
    
    MPI_Init(&argc,&argv);
    MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &p);
    f=p;

    /* set up the MPI description of MPI_Record */

    MPI_Address(&record, &baseaddress);
    lena[0] = 2; MPI_Address(&record.key,&loca[0]);
    loca[0] -= baseaddress; typa[0] = MPI_INT;
    lena[1] = 1; MPI_Address(&record.data   ,&loca[1]);
    loca[1] -= baseaddress; typa[1] = MPI_FLOAT;
    MPI_Type_struct(2, lena, loca, typa, &MPI_Record);
    MPI_Type_commit(&MPI_Record);

        /* create intercommunicators and set error handlers */
    if(my_rank==0) //manager process
    {
        my_comm = (MPI_Comm*) malloc ((p-1)*sizeof(MPI_Comm));
        for(i=1;i<p;i++)
        {
            MPI_Intercomm_create(MPI_COMM_SELF, 0, MPI_COMM_WORLD, i, IC_CREATE_TAG, &my_comm[i-1]);
            MPI_Comm_set_errhandler(my_comm[i-1], MPI_ERRORS_RETURN);
        }
    }
    else
    {
        my_comm = (MPI_Comm*)malloc(sizeof(MPI_Comm));
        MPI_Intercomm_create(MPI_COMM_SELF, 0, MPI_COMM_WORLD, 0, IC_CREATE_TAG, my_comm);
        MPI_Comm_set_errhandler(*my_comm, MPI_ERRORS_RETURN);
    }
    /* Each p has a link to manager and vice versa */

    /*END OF: MPI INIT*/
    
    if(argc>1) DATA_SIZE = atoi(argv[1]);
    
    if(my_rank==0)gettimeofday(&start,NULL);
    /* QUERY PROCESSING */ 
    if(my_rank != 0)
    {
        
        for(i=2;i<argc;i++)
            if(my_rank==atoi(argv[i])) exit(0);
            else f--;

        my_records=local_load("/tmp/pcc05/my_data",DATA_SIZE,"/tmp/pcc05/recovered_data",&size);
        
        //TODO: loop on queries...
        
        qsize=malloc(sizeof(int));
        
        while(1)
        {
            //recv query
            MPI_Recv(&query,1,MPI_Record,0,0,*my_comm,&status);
        
            //gather results
            if(query.key == -1)
            {
                //also get new size
                results = get_by_data(my_records, size, query.data, qsize);
            }
            else if(query.key != -2)
            {
                results = get_by_key(my_records, size, query.key, qsize);
            }
            else
                break;
            
            //printf("P%d Count = %d\n",my_rank,*qsize);
            //send query count
            MPI_Send(qsize,1,MPI_INT, 0, 1, *my_comm);
        
            //send results
            //MPI_Send(results,*qsize,MPI_Record,0,2,my_comm);
        }
        
    }
    else{
        for(i=2;i<argc;i++)
        {
            //MPI_Comm_free(&my_comm[atoi(argv[i])-1]);
            my_comm[atoi(argv[i])-1]=NULL;
            f--;
        }

        //GetQuery...
        /*
            Query Form
                MPI_Record: with either a key value (if != -1) or a data 
                value if key value does not exist.

                Query is always a greater than query returning all data
                that is greater than either of the two values depending on
                which one is being used
        */
        qcount=0;
        size = 0;
        qfile = fopen("/tmp/pcc05/query_data","r");
        while(fscanf(qfile,"%d,%f\n",&query.key,&query.data)!=EOF)
        {qcount++;
        //printf("NextQ %d,%f\t",query.key,query.data);
            //time(&start);
            //Send to all alive...
            for(i=1;i<p;i++)
                if(my_comm[i-1]!=NULL) MPI_Send(&query,1,MPI_Record,0,0,my_comm[i-1]);
        
            //Recieve counts
            qsize=malloc((p-1)*sizeof(int));
            for(i=1;i<p;i++)
                if(my_comm[i-1]!=NULL) 
                {
                    MPI_Recv(&qsize[i-1],1,MPI_INT,0,1,my_comm[i-1],&status);
                    size+=qsize[i-1];
                }
                else qsize[i-1]=-1;
            
            //time(&stop);
            //Get results
        //printf("TotalR %d\t Time %.6f\n",size,difftime(stop,start));
        }
        query.key=-2;
        for(i=1;i<p;i++)
            if(my_comm[i-1]!=NULL) MPI_Send(&query,1,MPI_Record,0,0,my_comm[i-1]);
        gettimeofday(&stop,NULL);
        printf("QP: Queries = %d, Live Nodes = %d, Avg Rec/Q = %d, Total time = %.4f\n",qcount,f,(size/qcount)+1,
                (stop.tv_sec-start.tv_sec)+
                ((float)(stop.tv_usec-start.tv_usec))/(1000000));
    }
    /* QUERY PROCESSED */
   
   /* Finalize 
    if(my_rank==0) 
        for (i=1; i<p; i++) 
            if(my_comm[i-1]) 
                MPI_Comm_free(&my_comm[i-1]);
    else MPI_Comm_free(my_comm);*/
    MPI_Finalize();
    exit(0);
}

Element* local_load(char *filename, int n, char *rfile, int *size)
{
    int i;
    FILE *file,*file2;
    struct stat buf;
    Element *array,*array2;
    
    file = fopen(filename,"r");
    file2 = fopen(rfile,"r");
    
    if(!file)return 0;
    array = (Element*)malloc((2*n)*sizeof(Element));
    for(i=0;i<n;i++)
        fscanf(file,"%d,%f\n",&(array[i].key),&(array[i].data));
    if(file2)
        while(fscanf(file2,"%d,%f\n",&(array[i].key),&(array[i].data))!=EOF)i++;
    *size=i+1;

    array2=(Element*)malloc((*size)*sizeof(Element));
    for(i=0;i<*size;i++) 
    {
        array2[i].key = array[i].key;
        array2[i].data = array[i].data;
    }
    
    free(array);
    return array2;
}

Element* get_by_key(Element *data, int size, int key, int* rsize)
{
    int *buf,i;
    Element* result;
    
    *rsize=0;
    buf = (int*)malloc(2*size*sizeof(int));
    for(i=0;i<size;i++)
        if(data[i].key >= key)
        {
            buf[*rsize]=i;
            (*rsize)++;
        }   
    
    result = (Element*)malloc((*rsize)*sizeof(Element));
    
    for(i=0;i<*rsize;i++)
    {
        result[i].key = data[buf[i]].key;
        result[i].data = data[buf[i]].data;
    }
    free(buf);
}

Element* get_by_data(Element *data, int size, int key, int* rsize)
{
    int *buf,i;
    Element* result;
    
    *rsize=0;
    buf = (int*)malloc(2*size*sizeof(int));
    for(i=0;i<size;i++)
        if(data[i].data >= key)
        {
            buf[*rsize]=i;
            (*rsize)++;
        }   
    
    result = (Element*)malloc((*rsize)*sizeof(Element));
 
    for(i=0;i<*rsize;i++)
    {
        result[i].key = data[buf[i]].key;
        result[i].data = data[buf[i]].data;
    }
    free(buf);
}
