#!/bin/bash

# This is your Grid Engine submission script. You may change several lines
# in this script
# 
# 1. The "#$ -pe mpi 8" line may be changed in order to run your code on
#     the appropriate number of processors.
# 
# 2. The "#$ -l r_ht=00:10:00" line should be changed to reflect the wall
#    clock time required by your job. You can also set a soft limit instead
#    of a hard limit by using "s_ht=1:00:00". This will attempt to run your
#    job for 1 hour by my kill it earlier for higher priority jobs.
# 
# 3. The "#$ -N rooster" may be changed to give your job a unique or
#    personalized name
# 
# 4. You can use the #$ -a MMDDhhmm" flag to run your job at a later time
# 
# 5. "#$ -V" will export all current environment variables, if you only
#    want to export a certain subset use "#$ -v VARIABLE1,VARIABLE2"
#    
# 6. The "mpirun -np $NSLOTS program_name" should be changed to execute a
#    different program. It is suggested to use the absolute path to the
#    progam, you can, however, enable option "#$ -cwd", which will execute
#    the script in the directory from which you have it submitted.
#
# The output will include a listing of the temporary environment that was
# set up for your program, as well as a record of all program output.
#
# Please do not change any of the other lines. Doing so will likely cause
# your submission to fail.
#
# Note: In the following section, all statements prefaced with #$ are
# interpreted as Grid Engine directives. They are only interpreted as
# comments if there are two or more # before the "$" flag. Grid Engine
# directives MUST always have one #.

# Name of my job (anything you want) 
# If you comment this out, the name will default to the name of the script.

#$ -N MPIFT 

# Setup a parallel environment with a given number of CPUs. 

#$ -pe lam 7

# Estimate of wallclock time for your program As a general rule, use about
# twice what you expect If you under-estimate, your job may be TERMINATED.
# Format = hours:minutes:seconds. 

#$ -l h_rt=00:10:00

# Schedule job for some time in the future. The format is MMDDhhmm
# (month/day/hour/minute). The following example submits a job that will be
# executed at 11:59 PM on December 31st. Uncomment this line if you want to
# use it. 

##$ -a 0

# Export your environment variables to the job

#$ -V

# If you want to merge standard output and standard error, enable this
# option.

##$ -j yes

# Specify the shell with which this script should be executed.
 
#$ -S /bin/bash

#$ -cwd

#########################################################################
#
# Run your parallel program here.
dir=output/out-TEST
size=0
qcount=2
i=1

mpirun -np $NSLOTS mk_tmp
mpirun -np $NSLOTS clean_tmp 
cp query_data /tmp/pcc05/query_data

mpirun -np $NSLOTS gethostnames 

size=`expr $size + 100000`

mpirun -np $NSLOTS init_data $size >$dir/$i.txt
mpirun -np $NSLOTS rs_parallel_encode >>$dir/$i.txt
mpirun -np $NSLOTS rs_dist_codes >>$dir/$i.txt

echo '---- INIT COMPLETE---' >>$dir/$i.txt

for (( j = 0; j<$qcount; j++ ))
do
    mpirun -np $NSLOTS query_processor $size >>$dir/$i.txt
done

echo '---KILLED Node 6 in [0,6]---' >>$dir/$i.txt

mpirun -np $NSLOTS rs_gather_codes 6 >>$dir/$i.txt
mpirun -np $NSLOTS rs_parallel_decode 6 >>$dir/$i.txt
mpirun -np $NSLOTS dist_records 6 >>$dir/$i.txt
#########################################################################
